#include <MIDI.h>
#include <DAC_MCP49xx.h>        // DAC lib

//Startup thing for //MIDI.h
MIDI_CREATE_DEFAULT_INSTANCE();

//Startup thing for DAC_MCP49xx.h
#define DACCS 10 // Arduino pin 10 / 328P pin 16 DACCS (chip select)
#define LDACS 7  // Arduino pin 7 / 328P pin 13 LDAC (latch enable)
DAC_MCP49xx dac(DAC_MCP49xx::MCP4922, DACCS, LDACS); // DAC initialize

#define groupPin1 3
#define groupPin2 4
#define groupPin3 5
#define groupPin4 6
#define groupPin5 7
#define groupPin6 8

#define keyPin1 A7
#define keyPin2 A6
#define keyPin3 A5
#define keyPin4 A4
#define keyPin5 A3
#define keyPin6 A2
#define keyPin7 A1

#define gate 2 /* will be 12 later */
/* #define triggerPin 12 /* external trigger input for sequencing / arpeggiator will come later */

uint8_t keydown = 0;


/* Variables related to the keyboard scan */
uint8_t keyPinCount;
uint8_t keyPinCountMin;
uint8_t keyPin[] = {keyPin7, keyPin1, keyPin2, keyPin3, keyPin4, keyPin5, keyPin6};
uint8_t groupPinCount;
uint8_t groupPin[] = {groupPin1, groupPin2, groupPin3, groupPin4, groupPin5, groupPin6};
uint8_t keyPressed[6][7];
uint8_t keyPressedPrev[6][7];

/* Variables for //MIDI In */
byte midinote;

/* Variables related to the CV generation in the DAC */
uint8_t usednote;
uint8_t octave;
uint8_t note;
//uint16_t  octDAC[] = { 128, 256, 512, 1024, 2048, 4095 };
uint16_t  octDAC[] = { 512, 1024, 2048, 4095, 2048, 4095 };
uint16_t noteDAC[] = { 2048, 2169, 2298, 2435, 2580, 2733, 2896, 3068, 3250, 3443, 3648, 3865, 4095 };


//*/
void setup() {
  MIDI.begin(); // //MIDI receive on default channel 1
  MIDI.turnThruOff();

  //Serial.begin(38400);

  pinMode(groupPin1, OUTPUT);
  pinMode(groupPin2, OUTPUT);
  pinMode(groupPin3, OUTPUT);
  pinMode(groupPin4, OUTPUT);
  pinMode(groupPin5, OUTPUT);
  pinMode(groupPin6, OUTPUT);

  pinMode(keyPin1, INPUT_PULLUP);
  pinMode(keyPin2, INPUT_PULLUP);
  pinMode(keyPin3, INPUT_PULLUP);
  pinMode(keyPin4, INPUT_PULLUP);
  pinMode(keyPin5, INPUT_PULLUP);
  pinMode(keyPin6, INPUT_PULLUP);
  pinMode(keyPin7, INPUT_PULLUP);

  pinMode(gate, OUTPUT);

  for (groupPinCount = 0; groupPinCount < 6; ++groupPinCount) {
    for (keyPinCount = 0; keyPinCount < 7; ++keyPinCount) {
      keyPressed[groupPinCount][keyPinCount] = 0;
      keyPressedPrev[groupPinCount][keyPinCount] = 0;
    }
  }



  /* Serial.print FIRMWARE VERSION */
  //Serial.println(F("notetest 1"));
}


void loop() {
  /* */
  if (MIDI.read()) {
    switch (MIDI.getType()) {
      case midi::NoteOn:
        midinote = MIDI.getData1();
        if (midinote >= 48) {
          usednote = midinote - 48;
        } else {
          usednote = 0;
        }

        if (usednote >= 0 && usednote <= 11) {
          octave = 0;
        }
        if (usednote >= 12 && usednote <= 23) {
          octave = 1;
        }
        if (usednote >= 24 && usednote <= 35) {
          octave = 2;
        }
        if (usednote >= 36 && usednote <= 47) {
          octave = 3;
        }
        if (usednote >= 48 && usednote <= 59) {
          octave = 4;
        }
        if (usednote > 59) {
          octave = 5;
        }
        note = usednote % 12;

        dac.output2(octDAC[octave], noteDAC[note]);
        if (keydown == 0) keydown = 1;
        break;
      case midi::NoteOff:
        if (keydown > 0) keydown = 0;
        break;
    }
  }

  /**/

  for (groupPinCount = 0; groupPinCount < 6; ++groupPinCount) {
    digitalWrite(groupPin[groupPinCount], HIGH);
    if (groupPinCount == 0) {
      keyPinCountMin = 0;
    } else {
      keyPinCountMin = 1;
    }
    for (keyPinCount = keyPinCountMin; keyPinCount < 7; ++keyPinCount) {
      if ( analogRead(keyPin[keyPinCount]) > 512) {
        keyPressed[groupPinCount][keyPinCount] = 1;
      } else {
        keyPressed[groupPinCount][keyPinCount] = 0;
      }
      if (keyPressed[groupPinCount][keyPinCount] == 1 && keyPressedPrev[groupPinCount][keyPinCount] == 0) {
        ++keydown;
        keyPressedPrev[groupPinCount][keyPinCount] = 1;
        Serial.print(F("NoteOn keyPinCount="));
        Serial.print(keyPinCount);
        Serial.print(F(" groupPinCount="));
        Serial.print(groupPinCount);
        Serial.print(F(" NoteOn="));
        Serial.print((groupPinCount * 6) + (keyPinCount));
        Serial.print(F(" playedNotes="));
        Serial.print(keydown);
        usednote = ((groupPinCount * 6) + keyPinCount);
        if (usednote >= 0 && usednote <= 11) {
          octave = 0;
        }
        if (usednote >= 12 && usednote <= 23) {
          octave = 1;
        }
        if (usednote >= 24 && usednote <= 35) {
          octave = 2;
        }
        if (usednote >= 36 && usednote <= 47) {
          octave = 3;
        }
        if (usednote >= 48 && usednote <= 59) {
          octave = 4;
        }
        if (usednote > 59) {
          octave = 5;
        }
        note = usednote % 12;

        dac.output2(octDAC[octave], noteDAC[note]);

/*
        Serial.print(F(" usednote="));
        Serial.print(usednote);
        Serial.print(F(" octave="));
        Serial.print(octave);
        Serial.print(F(" note="));
        Serial.print(note);
        Serial.print(F(" oD="));
        Serial.print(octDAC[octave]);
        Serial.print(F(" nD="));
        Serial.print(noteDAC[note]);
        Serial.println(F(" "));
*/


      }
      
      if (keyPressed[groupPinCount][keyPinCount] == 0 && keyPressedPrev[groupPinCount][keyPinCount] == 1) {
        --keydown;
        keyPressedPrev[groupPinCount][keyPinCount] = 0;
/*
        Serial.print(F("NoteOff keyPinCount="));
        Serial.print(keyPinCount);
        Serial.print(F(" groupPinCount="));
        Serial.print(groupPinCount);
        Serial.print(F("NoteOff="));
        Serial.print((groupPinCount * 6) + (keyPinCount));
        Serial.print(F(" playedNotes="));
        Serial.println(keydown);
*/
      }
    }
    digitalWrite(groupPin[groupPinCount], LOW);

  }
  if (keydown == 0) {
    digitalWrite(gate, LOW);
  } else {
    digitalWrite(gate, HIGH);
  }


}
